//---------------------------------------------------------------------------
// Copyright (C) 2000 Dallas Semiconductor Corporation, All Rights Reserved.
// 
// Permission is hereby granted, free of charge, to any person obtaining a 
// copy of this software and associated documentation files (the "Software"), 
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the 
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included 
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
// IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES 
// OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
// OTHER DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of Dallas Semiconductor 
// shall not be used except as stated in the Dallas Semiconductor 
// Branding Policy. 
//---------------------------------------------------------------------------
//
// owdigpot.C - This utility uses TMEX to DS2890.
//
// Compiler:  
// Version: 1.00
//
//

#include "ibtmexcw.H"   

// defines
#define CHARGE_PUMP_OFF 0x0C
#define CHARGE_PUMP_ON  0x2C

// Local Function Prototypes 
short FindFirstFamily(short);
short ReadPotInfo(uchar *, uchar *);
void PrintPotInfo(uchar, uchar);
uchar dowcrc(uchar);
void PrintHeader(char *);
short PotAdjust(short,uchar *);
short PotSet(uchar);
short WritePotControl(uchar);
void Exit_Prog(char *, short);
short Aquire1WireNet(void);
void Release1WireNet(void);
int EnterNum(char *, int, long *, long, long);

// Global Variables to hold TMEX state 
short PortType, PortNum;
long session_handle;                         
uchar state_buffer[5120];

//----------------------------------------------------------------------
//  This is the Main routine for OWDigPot
// 
void main(short argc, char **argv)
{
   short found,done;
   char ch;       
   uchar control,position;
   long getnum;

   // check arguments 
   if (argc > 1)
   {
       printf("\nusage: owdigpot\n"
              "  - read and change the settings on a DS2890 digital potentiometer\n"
              "  - version 1.00\n");
       Exit_Prog("",BAD_ARGUMENT);
   }

   // read the default PortNum and PortType 
   if (!TMReadDefaultPort(&PortNum,&PortType))
   {
       printf("ERROR, Could not read the default PortNum and PortType!\n");
       exit(1);
   }

   // print the header info 
   PrintHeader("OwDigPot Version 1.00");

   // find the device   
   found = FALSE;
   if (Aquire1WireNet())
   {
      // setup the MicroLan 
      if (TMSetup(session_handle) == 1)
      {
         printf("Searching for a DS2890..\n\n");

         // look for the first switch iButton (family type 0x2C) 
         if (FindFirstFamily(0x2C))
            found = TRUE;
         // Temperature iButton not on MicroLan 
         else   
            printf("DS2890 not found on MicroLan!"); 
      }   
      // MicroLan port not valid 
      else   
         printf("MicroLan setup failed!"); 
      // release port 
      Release1WireNet();
   }

   // continue if a DS2890 was found 
   if (found)                          
   {  
      // read and print status
      ReadPotInfo(&control, &position);
      PrintPotInfo(control, position);
      
      // loop while not done 
      done = FALSE;
      do
      {  
         // print menu 
         printf("\n\n(1) Display the 1-Wire Potentiometer Info\n"
              "(2) Increment wiper\n"
              "(3) Decrement wiper\n"
              "(4) Set wiper value\n"
              "(5) Step wiper to max\n"
              "(6) Step wiper to min\n"
              "(7) Charge pump ON\n"
              "(8) Charge pump OFF\n"
              "(9) Quit\n"
              "Select a Number:");
         ch = getche();
         printf("\n\n");
         
         // do something from the menu selection              
         switch(ch)
         {        
            case '1': 
               if (ReadPotInfo(&control, &position))
                  PrintPotInfo(control, position);
               else
                  printf("\nERROR reading info!\n");
               break;
            // Increment wiper
            case '2':
               if (PotAdjust(1,&position))
                  PrintPotInfo(control, position);
               else
                  printf("\nERROR adjusting wiper!\n");
               break;
            // Decrement wiper
            case '3':
               if (PotAdjust(0,&position))
                  PrintPotInfo(control, position);
               else
                  printf("\nERROR adjusting wiper!\n");
               break;
            // set wiper value
            case '4':
               getnum = position;
               if (EnterNum("Enter a new wiper position (0 to 255):",
                      3, &getnum, 0, 255))
               {
                  if (PotSet((uchar)getnum))
                  {
                     position = (uchar)getnum;
                     PrintPotInfo(control, position);
                  }
                  else
                     printf("\nERROR setting wiper!\n");
               }
               else
                  printf("Abort setting wiper!\n");
               break;
            // Step wiper to max
            case '5': 
               while (position < 0xFF)
               {
                  if (PotAdjust(1,&position))
                     PrintPotInfo(control,position);
                  else
                  {
                     printf("\nERROR incrementing wiper!\n");
                     break;
                  }
               }
               break; 
            // Step wiper to min
            case '6':
               while (position > 0)
               {
                  if (PotAdjust(0,&position))
                     PrintPotInfo(control,position);
                  else
                  {
                     printf("\nERROR incrementing wiper!\n");
                     break;
                  }
               }
               break;
            // Charge pump ON
            case '7':
               if (WritePotControl(CHARGE_PUMP_ON))
               {
                  if (ReadPotInfo(&control, &position))
                     PrintPotInfo(control, position);
                  else
                     printf("\nERROR reading info!\n");
               }
               else
                  printf("\nERROR writing control!\n");
               break;
            // Charge pump OFF
            case '8':
               if (WritePotControl(CHARGE_PUMP_OFF))
               {
                  if (ReadPotInfo(&control, &position))
                     PrintPotInfo(control, position);
                  else
                     printf("\nERROR reading info!\n");
               }
               else
                  printf("\nERROR writing control!\n");
               break;
            // Quit
            case '9': case 'q': case 'Q':
               done = TRUE;  
               break;                            
            default:
               MessageBeep(MB_ICONEXCLAMATION);
         };

      }
      while (!done);
   }
}

//----------------------------------------------------------------------
// Find the first device with a particular family
// 
short FindFirstFamily(short family)
{
   short j;
   short ROM[9];
   
   // set up to find the first device with family 'family 
   if (TMFamilySearchSetup(session_handle,&state_buffer,family) == 1)
   { 
      // get first device in list with the specified family 
      if (TMNext(session_handle,state_buffer) == 1)
      {
         // read the rom number 
         ROM[0] = 0;
         TMRom(session_handle,state_buffer,ROM);   
         
         // check if correct type 
         if ((family & 0x7F) == (ROM[0] & 0x7F))
         {
            printf("Serial ROM ID: ");  
            for (j = 7; j >= 0; j--)
               printf("%02X",ROM[j]);
            printf("\n\n");
            return 1;
         }        
      }
   }
     
   // failed to find device of that type 
   return 0;                              
}

//----------------------------------------------------------------------
// Read the pot control and position.
//
// Return TRUE  (1) on success
//        FALSE (0) if failed to read
//
short ReadPotInfo(uchar *control, uchar *position)
{
   short rt=0;
   uchar pbuf[40];

   if (Aquire1WireNet())
   {
      // check for alarm indicator 
      if (TMAccess(session_handle,state_buffer) > 0) 
      {
         // construct a packet to send  
         pbuf[0] = 0xF0; // read position command 
         pbuf[1] = 0xFF; // control 
         pbuf[2] = 0xFF; // position 
         pbuf[3] = 0xFF; // confirm get 0's 

         // perform the block 
         if (TMBlockStream(session_handle,pbuf,4) == 4)
         {
            // check end 0's
            if (pbuf[3] == 0)
            {
               *control = pbuf[1];
               *position = pbuf[2];
               rt = TRUE;
            }
         }
      }

      // release port 
      Release1WireNet();
   }

   return rt;
}

//----------------------------------------------------------------------
// Wrote pot control byte
//
// Return TRUE  (1) on success
//        FALSE (0) if failed to write control
//
short WritePotControl(uchar control)
{
   short rt=0;
   uchar pbuf[40];

   if (Aquire1WireNet())
   {
      // check for alarm indicator 
      if (TMAccess(session_handle,state_buffer) > 0) 
      {
         // construct a packet to send  
         pbuf[0] = 0x55;    // write control reg 
         pbuf[1] = control; // control 
         pbuf[2] = 0xFF;    // control echo 

         // perform the block 
         if (TMBlockStream(session_handle,pbuf,3) == 3)
         {
            // check echo
            if (pbuf[2] == control)
            {
               // set the release code
               pbuf[0] = 0x96;
               pbuf[1] = 0xFF;
               pbuf[2] = 0xFF;  // should be zeros

               // perform the block 
               if (TMBlockStream(session_handle,pbuf,3) == 3)
               {
                  if (pbuf[2] == 0x00)
                     rt = TRUE;
               }
            }
         }
      }

      // release port 
      Release1WireNet();
   }

   return rt;
}

//----------------------------------------------------------------------
// Change the wiper position by 1.  If 'direction' is != 0 then add
// 1 to wiper position, else subract 1.  Pass a pointer to the last
// position to verify new position.  Change 'last_position' to 
// the new position.
//
// Return TRUE  (1) on success
//        FALSE (0) if failed to increment or decrement
//                                                           
short PotAdjust(short direction, uchar *last_position)
{
   short rt=0;
   uchar new_position;
   uchar pbuf[40];

   if (Aquire1WireNet())
   {
      // check for alarm indicator 
      if (TMAccess(session_handle,state_buffer) > 0) 
      {
         // construct a packet to send  
         if (direction)
         {
            pbuf[0] = 0xC3; // increment
            if (*last_position != 0xFF)
               new_position = *last_position + 1;
            else
               new_position = 0xFF;
         }
         else
         {
            pbuf[0] = 0x99; // deccrement
            if (*last_position > 0)
               new_position = *last_position - 1;
            else
               new_position = 0;
         }
 
         pbuf[1] = 0xFF; // new position read 

         // perform the block 
         if (TMBlockStream(session_handle,pbuf,2) == 2)
         {
            if (((new_position == 0xF0) && (pbuf[1] == 0xFC)) ||
                     ((new_position == 0xAA) && (pbuf[1] == 0xFB)))
            {
               printf("Detected REV1 bug, (new position is %02X but got echo of %02X)\n",
                      new_position, pbuf[1]);
               *last_position = new_position;
               rt = TRUE;
            }
            else if (new_position == pbuf[1])
            {
               *last_position = new_position;
               rt = TRUE;
            }
         }
      }
      // release port 
      Release1WireNet();
   }

   return rt;
}

//----------------------------------------------------------------------
// Set the wiper position
//                                                           
// Return TRUE  (1) on success
//        FALSE (0) if failed to set
//
short PotSet(uchar new_position)
{
   short rt=0;
   uchar pbuf[40];

   if (Aquire1WireNet())
   {
      // check for alarm indicator 
      if (TMAccess(session_handle,state_buffer) > 0) 
      {
         pbuf[0] = 0x0F; // write position
         pbuf[1] = new_position; 
         pbuf[2] = 0xFF; // echo new position read 

         // perform the block 
         if (TMBlockStream(session_handle,pbuf,3) == 3)
         {
            if (pbuf[2] == new_position)
            {
               // set the release code
               pbuf[0] = 0x96;
               pbuf[1] = 0xFF;
               pbuf[2] = 0xFF;  // should be zeros

               // perform the block 
               if (TMBlockStream(session_handle,pbuf,3) == 3)
               {
                  if (pbuf[2] == 0x00)
                     rt = TRUE;
               }
            }
         }
      }
      // release port 
      Release1WireNet();
   }

   return rt;
}

//----------------------------------------------------------------------
// Print the pot information .
//                                                           
void PrintPotInfo(uchar control, uchar position)
{ 
   printf("-------------------------\n");
   printf("Wiper position: %d\n",position);
   if (control == CHARGE_PUMP_ON)
      printf("Charge pump: ON\n");
   else
      printf("Charge pump: OFF\n");
   printf("-------------------------\n");
}

//--------------------------------------------------------------------------
//  Prints a message, and the current driver versions.
//
void PrintHeader(char *msg)
{
   char tstr[80];       

   // print a header with driver version info   
   printf("\n%s\n"
         "Dallas Semiconductor Corporation\n"
         "Copyright (C) 1992-2000\n\n",msg);
   printf("Port number: %d    Port type: %d\n",PortNum,PortType);
   Get_Version(tstr);
   printf("Main driver version: %s\n",tstr);
   printf("Sub-driver version, Type%d:",PortType);
   if (TMGetTypeVersion(PortType,tstr) < 0)
   {
      printf("No Hardware Driver for this type found!");
      exit(1);
   }
   printf(" %s\n\n\n",tstr);
}

//----------------------------------------------------------------------
// Aquire a session on the 1-Wire Net
// return TRUE(1)  : session on 1-Wire Net aquired
//        FALSE(0) : could not aquire a session on the 1-Wire Net
//                                                          
short Aquire1WireNet(void)
{
   short printmsg=1;
   ulong tm;

   // set timeout
   tm = GetTickCount() + 2000;

   // loop to get a session handle or timeout 
   do
   {
      // attempt to get a session 
      session_handle = TMExtendedStartSession(PortNum,PortType,NULL);
      if (session_handle > 0)  
         return TRUE;
      else if (session_handle == 0)
      {
         if (printmsg)
         {
            printf("Waiting to get access to 1-Wire Net");
            printmsg = 0;
         }
      }
      else if (session_handle < 0)   
      {
         printf("No Hardware Driver for this type found!");
         return FALSE;
      }
   }
   while (tm > GetTickCount());

   return FALSE;   
}

//----------------------------------------------------------------------
// Give up the session on the 1-Wire Net
//                                                          
void Release1WireNet(void)
{
   // end the session if one is open
   TMEndSession(session_handle);
}

//--------------------------------------------------------------------------
//  Prints a message, beeps and exits the program.
// 
void Exit_Prog(char *msg, short errcd)
{
     // print the message left justified 
     printf("\r%s",msg);

     //  error beep if return code is not 0 
     if (errcd != NORMAL_EXIT)
        MessageBeep(MB_ICONEXCLAMATION);

     //  return error code passed 
     exit(errcd);
}

//----------------------------------------------------------------------
// Enter a decimal string and convert it to an unsigned long
// Prompt again if not within min/max inclusive.
//                                                          
int EnterNum(char *msg, int numchars, long *value, long min, long max)
{
   short tmp,cnt,isneg=FALSE; 
   char ch;

   // loop while not in correct range
   do
   {
      MessageBeep(MB_OK);
      printf("%s (%d): ",msg,*value);

      // loop for each character read
      cnt = 0;
      for (;;)
      {
         ch = getch();

         // negative flag
         if (ch == '-')
         {
            if (!isneg)
            {
               isneg = TRUE;
               printf("-");
               cnt++;
            }
         }
         // backspace 
         if (ch == 0x08)
         {
            if (cnt)
            {
               if (isneg && (cnt == 1))
                  isneg = FALSE;
               else
                  *value /= 10;
               printf("%c %c",ch,ch);
               cnt--;
            } 
         }
         // escape 
         if (ch == 0x1B)
         {
            printf("  Aborted\n\n");
            return FALSE;
         }
         // enter 
         if (ch == 0x0D)
         {
            printf("\n");
            break;
         }
         // number 
         else if ((ch >= '0') && (ch <= '9'))
         {
            if (cnt == 0)
               *value = 0;

            if (cnt < numchars)
            {
               printf("%c",ch);
               tmp = ch - 0x30;
               *value *= 10; 
               *value += tmp;
               cnt++;
            }
         }
      }

      if (isneg)
         *value = -*value;
   }
   while ((*value < min) || (*value > max));

   return TRUE;
}
